<?php
if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/*
* Plugin Name: Analytify - WooCommerce add-on
* Plugin URI: https://analytify.io/enhanced-ecommerce-google-analytics-for-woocommerce/
* Description: This Add-on will track the e-commerce behaviour of users, digital Sales, Transactions and Revenues in Google Analytics and Displays Stats in a unique and intiutive way which is very understandable even for non-technical WordPress users.
* Version: 2.0.12
* Author: Team Analytify
* Author URI: https://analytify.io/
* Developer: Analytify LLC
* Developer URI: https://analytify.io/
* License: GPLv2+
* Min WP Version: 3.0
* Max WP Version: 5.1
* Text Domain: wp-analytify-woocommerce
* Domain Path: /languages
* WC requires at least: 2.5
* WC tested up to: 3.5.0
*/

define( 'ANALTYIFY_WOOCOMMERCE_VERSION', '2.0.12' );
define( 'ANALYTIFY_WOOCOMMERCE_UPGRADE_PATH', __FILE__ );

function wpa_check_woo_admin_notice() {
  $class = "error";
  $message = esc_html__( 'Analytify for WooCommerce works with WooCommerce plugin, Please activate WooCommerce.', 'wp-analytify-woocommerce' );
  echo"<div class=\"$class\"> <p>$message</p></div>";
}

function wpa_check_analytify_admin_notice() {
  $class = "error";
  $message = esc_html__( 'Analytify for WooCommerce works with the Analytify PRO plugin. Please activate Analytify PRO.', 'wp-analytify-woocommerce' );
  echo"<div class=\"$class\"> <p>$message</p></div>";
}



add_action( 'plugins_loaded', 'load_analytify_woo', 20 );

function load_analytify_woo() {

  if ( ! class_exists( 'WP_Analytify' ) ) {
		return;
	}


  /**
   * Check if WooCommerce is active or not
   */
  if ( ! class_exists( 'WooCommerce' ) ) {

    add_action( 'admin_notices', 'wpa_check_woo_admin_notice' );
    return;
  }

  /**
   * Check if Analytify is active
   */
  if ( ! class_exists( 'WP_Analytify_Pro' ) ) {

    add_action( 'admin_notices', 'wpa_check_analytify_admin_notice' );
    return;
  }


  $abspath = dirname( __FILE__ );
  include_once $abspath . '/class-wp-analytify-woo.php';

  /**
  * Returns the main instance of WP_Analytify_WooCommerce to prevent the need to use globals.
  *
  * @since  1.0.0
  * @return WP_Analytify_WooCommerce
  */
  function WPAnalytifyWooCommerce() {
    return WP_Analytify_WooCommerce::instance();
  }

  // Global for backwards compatibility.
  $GLOBALS['WPAnalytifyWooCommerce'] = WPAnalytifyWooCommerce();

}


/**
 * Load TextDoamin
 *
 * @since 1.0.2
 */
function wp_analytify_woo_load_text_domain() {
	$plugin_dir = basename( dirname( __FILE__ ) );
	load_plugin_textdomain( 'wp-analytify-woocommerce', false , $plugin_dir . '/languages/' );
}
add_action( 'init', 'wp_analytify_woo_load_text_domain' );


/**
 * Helper function for translation.
 */
if ( ! function_exists( 'analytify__' ) ) {
	/**
	 * Wrapper for __() gettext function.
	 * @param  string $string     Translatable text string
	 * @param  string $textdomain Text domain, default: wp-analytify
	 * @return void
	 */
	function analytify__( $string, $textdomain = 'wp-analytify' ) {
		return __( $string, $textdomain );
	}
}
if ( ! function_exists( 'analytify_e' ) ) {
	/**
	 * Wrapper for _e() gettext function.
	 * @param  string $string     Translatable text string
	 * @param  string $textdomain Text domain, default: wp-analytify
	 * @return void
	 */
	function analytify_e( $string, $textdomain = 'wp-analytify' ) {
		echo __( $string, $textdomain );
	}
}
